/*
 * Copyright (C) 2025 The Phosh.mobi e.V.
 *
 * SPDX-License-Identifier: GPL-3.0+
 */

#pragma once

#include "lcb-message.h"

G_BEGIN_DECLS

#define CBD_TYPE_MESSAGE (cbd_message_get_type ())

G_DECLARE_FINAL_TYPE (CbdMessage, cbd_message, CBD, MESSAGE, LcbMessage);

CbdMessage    *cbd_message_new             (const char      *text,
                                            uint             channel,
                                            uint             message_code,
                                            uint             update,
                                            gint64           timestamp,
                                            LcbSeverityLevel severity,
                                            const char      *severity_subject,
                                            const char      *operator_code);

G_END_DECLS
