/*
 * Copyright (C) 2025 The Phosh.mobi e.V.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include <libmm-glib.h>

G_BEGIN_DECLS

#define CBD_TYPE_MODEM_3GPP (cbd_modem_3gpp_get_type ())

G_DECLARE_FINAL_TYPE (CbdModem3gpp, cbd_modem_3gpp, CBD, MODEM_3GPP, GObject)

CbdModem3gpp *cbd_modem_3gpp_new (MMModem3gpp *modem_3gpp);
gboolean      cbd_modem_3gpp_match(CbdModem3gpp *self, MMModem3gpp *modem_3gpp);
const char*   cbd_modem_3gpp_get_country (CbdModem3gpp *self);
const char*   cbd_modem_3gpp_get_operator_code (CbdModem3gpp *self);

G_END_DECLS
