/*
 * Copyright (C) 2025 The Phosh.mobi e.V.
 *
 * SPDX-License-Identifier: GPL-3.0+
 *
 * Author: Evangelos Ribeiro Tzaras <devrtz@fortysixandtwo.eu>
 */

#define G_LOG_DOMAIN "cbd"

#include "cbd-daemon.h"

int
main (int argc, char *argv[])
{
  g_autoptr (CbdDaemon) daemon = cbd_daemon_new ();

  return g_application_run (G_APPLICATION (daemon), argc, argv);
}
