/*
 * Copyright (C) 2025 The Phosh.mobi e.V.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include "cbd-channel-manager.h"

#include <gio/gio.h>

G_BEGIN_DECLS

#define CBD_TYPE_DAEMON (cbd_daemon_get_type ())

G_DECLARE_FINAL_TYPE (CbdDaemon, cbd_daemon, CBD, DAEMON, GApplication)

CbdDaemon *cbd_daemon_new (void);

G_END_DECLS
