/*
 * Copyright (C) 2025 The Phosh.mobi e.V.
 *
 * SPDX-License-Identifier: LGPL-2.1+
 */
#pragma once

#if !defined (__LIBCELLBROADCAST_H_INSIDE__) && !defined (LIBCELLBROADCAST_COMPILATION)
#error "Only <libcellbroadcast.h> can be included directly."
#endif

#include "lcb-dbus.h"

#include <glib.h>


G_BEGIN_DECLS

gboolean                lcb_init                (GError **error);
void                    lcb_uninit              (void);
gboolean                lcb_is_initted          (void);
LcbDBusCbd             *lcb_get_proxy           (void);

G_END_DECLS
