/*
 * Copyright (C) 2025 The Phosh.mobi e.V.
 *
 * SPDX-License-Identifier: LGPL-2.1+
 *
 * Author: Evangelos Ribeiro Tzaras <devrtz@fortysixandtwo.eu>
 */

#pragma once

#if !defined (__LIBCELLBROADCAST_H_INSIDE__) && !defined (LIBCELLBROADCAST_COMPILATION)
#error "Only <libcellbroadcast.h> can be included directly."
#endif

#include "lcb-util.h"

G_BEGIN_DECLS

gboolean          lcb_util_save_messages_to_file          (GListModel *messages,
                                                           const char *path,
                                                           GError    **error);

G_END_DECLS
