/*
 * Copyright (C) 2025 The Phosh.mobi e.V.
 *
 * SPDX-License-Identifier: GPL-3.0+
 */

#pragma once

#include "lcb-dbus.h"

#include <glib-object.h>

G_BEGIN_DECLS

#define CBD_TYPE_MM_MANAGER (cbd_mm_manager_get_type ())

G_DECLARE_FINAL_TYPE (CbdMmManager, cbd_mm_manager, CBD, MM_MANAGER, GObject);

CbdMmManager      *cbd_mm_manager_new     (void);
CbdChannelManager *cbd_mm_manager_get_channel_manager (CbdMmManager *self);

G_END_DECLS
