/*
 * Copyright (C) 2025 The Phosh.mobi e.V.
 *
 * SPDX-License-Identifier: LGPL-2.1+
 */
#pragma once

#include <glib.h>

G_BEGIN_DECLS

#define __LIBCELLBROADCAST_H_INSIDE__

#ifndef LIBCELLBROADCAST_USE_UNSTABLE_API
#error    LIBCELLBROADCAST is unstable API. You must define LIBCELLBROADCAST_USE_UNSTABLE_API before including libcellbroadcast.h
#endif

#include "lcb-cbd.h"
#include "lcb-dbus.h"
#include "lcb-enum-types.h"
#include "lcb-main.h"
#include "lcb-message.h"
#include "lcb-util.h"

G_END_DECLS
