/*
 * Copyright (C) 2025 The Phosh.mobi e.V.
 *
 * SPDX-License-Identifier: LGPL-2.1+
 *
 * Author: Evangelos Ribeiro Tzaras <devrtz@fortysixandtwo.eu>
 */

#pragma once

#include <gio/gio.h>

#if !defined (__LIBCELLBROADCAST_H_INSIDE__) && !defined (LIBCELLBROADCAST_COMPILATION)
#error "Only <libcellbroadcast.h> can be included directly."
#endif

G_BEGIN_DECLS

void            lcb_cbd_get_messages               (GCancellable       *cancellable,
                                                    GAsyncReadyCallback callback,
                                                    gpointer            user_data);
GListModel     *lcb_cbd_get_messages_finish        (GAsyncResult *result, GError **error);
GListModel     *lcb_cbd_get_messages_sync          (GError **error);

G_END_DECLS
