/*
 * Copyright (C) 2025 The Phosh.mobi e.V.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include <libmm-glib.h>

G_BEGIN_DECLS

#define CBD_TYPE_MODEM_CELL_BROADCAST (cbd_modem_cell_broadcast_get_type ())

G_DECLARE_FINAL_TYPE (CbdModemCellBroadcast, cbd_modem_cell_broadcast, CBD, MODEM_CELL_BROADCAST, GObject)

CbdModemCellBroadcast *cbd_modem_cell_broadcast_new   (MMModemCellBroadcast  *modem_cell_broadcast);
gboolean               cbd_modem_cell_broadcast_match (CbdModemCellBroadcast *self,
                                                       MMModemCellBroadcast  *modem_cell_broadcast);
void                   cbd_modem_cell_broadcast_set_channels (CbdModemCellBroadcast *self,
                                                              GArray                *channels);

G_END_DECLS
